package pboweaverplugin;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import pboweaverplugin.builder.WeaverNature;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {
	private class PbOProjectDeletedChangedListener implements IResourceChangeListener {

		@Override
		public void resourceChanged(IResourceChangeEvent event) {
			System.out.println("In Listener class");
			
			IResource res = event.getResource();
			if (!(res instanceof IProject)) return;
			
			final IProject project = (IProject)res;
			
			try { 
				//WeaverNature.removePbONature(project, null); 
//				String[] natures = project.getDescription().getNatureIds();
//				for(String nature: natures) {
//					System.out.println("Looking at nature: "+nature);
//					if (nature.equals(WeaverNature.NATURE_ID)) {
//						String resourcesLocation = ResourcesPlugin.getPlugin().getStateLocation().toString();
//						resourcesLocation += File.separatorChar + project.getName();
//						File f = new File(resourcesLocation,"sources.dat");
//						System.out.println("Looking at file(exists? "+f.exists()+"): "+f);
//						if (f.exists()) { f.delete(); }
//						System.out.println("sources.dat removed");
//					}
//				}
				
				IWorkspaceRunnable runnable = new IWorkspaceRunnable() {

					@Override
					public void run(IProgressMonitor monitor)
							throws CoreException {
						String resourcesLocation = ResourcesPlugin.getPlugin().getStateLocation().toString();
						resourcesLocation += File.separatorChar + project.getName();
						File f = new File(resourcesLocation,"sources.dat");
						if (f.exists()) { f.delete(); }
						
					}
					
				};
				
				ResourcesPlugin.getWorkspace().run(runnable, new NullProgressMonitor());
				
			} catch (CoreException ex) {
				System.out.println("Could not remove pbo nature from project");
				ex.printStackTrace();
			}
			
		}
		
	}

	// The plug-in ID
	public static final String PLUGIN_ID = "PbOWeaverPlugin"; //$NON-NLS-1$

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
		ResourcesPlugin.getWorkspace()
			.addResourceChangeListener(new PbOProjectDeletedChangedListener(), IResourceChangeEvent.PRE_DELETE);
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
}
