package pboweaverplugin.builder;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

public class WeaverNature implements IProjectNature {

	/**
	 * ID of this project nature
	 */
	public static final String NATURE_ID = "PbOWeaverPlugin.sampleNature";

	private IProject project;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#configure()
	 */
	public void configure() throws CoreException {
		IProjectDescription desc = project.getDescription();
		ICommand[] commands = desc.getBuildSpec();

		for (int i = 0; i < commands.length; ++i) {
			if (commands[i].getBuilderName().equals(WeaverBuilder.BUILDER_ID)) {
				return;
			}
		}

		ICommand[] newCommands = new ICommand[commands.length + 1];
		//System.arraycopy(commands, 0, newCommands, 0, commands.length);
		System.arraycopy(commands, 0, newCommands, 1, commands.length);
		ICommand command = desc.newCommand();
		command.setBuilderName(WeaverBuilder.BUILDER_ID);
		//newCommands[newCommands.length - 1] = command;
		newCommands[0] = command;
		desc.setBuildSpec(newCommands);
		project.setDescription(desc, null);
		
		//addPbONature(project,null);
		
		addPbONature(getProject(),null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
		IProjectDescription description = project.getDescription();
		ICommand[] commands = description.getBuildSpec();
		for (int i = 0; i < commands.length; ++i) {
			if (commands[i].getBuilderName().equals(WeaverBuilder.BUILDER_ID)) {
				ICommand[] newCommands = new ICommand[commands.length - 1];
				System.arraycopy(commands, 0, newCommands, 0, i);
				System.arraycopy(commands, i + 1, newCommands, i,
						commands.length - i - 1);
				description.setBuildSpec(newCommands);
				project.setDescription(description, null);	
				
				removePbONature(project, null);
				
				return;
			}
		}
		
		//removePbONature(getProject(),null);
		
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#getProject()
	 */
	public IProject getProject() {
		return project;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
	 */
	public void setProject(IProject project) {
		this.project = project;
	}
	
	public static void addPbONature(IProject project, IProgressMonitor mon) throws CoreException {
		addPboNature(project, NATURE_ID, mon);
	}
	
	public static void removePbONature(IProject project, IProgressMonitor mon) throws CoreException {
		removePbONature(project, NATURE_ID, mon);
	}
	
	private static void addPboNature(IProject project, String natureId, IProgressMonitor monitor) throws CoreException {
		try {
	        if (monitor == null) {
	            monitor = new NullProgressMonitor();
	        }
    		IProjectDescription description = project.getDescription();
    		String[] prevNatures = description.getNatureIds();
    		for (String prevNature : prevNatures) {
    			if (natureId.equals(prevNature))
    				return;
    		}
    		String[] newNatures = new String[prevNatures.length + 1];
    		System.arraycopy(prevNatures, 0, newNatures, 0, prevNatures.length);
    		newNatures[prevNatures.length] = natureId;
    		description.setNatureIds(newNatures);
    		project.setDescription(description, monitor);
		}
		finally {
			monitor.done();
		}
	}
	
	private static void removePbONature(IProject project, String natureId, IProgressMonitor monitor) throws CoreException {
		IProjectDescription description = project.getDescription();
		String[] prevNatures = description.getNatureIds();
		List<String> newNatures = new ArrayList<String>(Arrays.asList(prevNatures));
		newNatures.remove(natureId);
		description.setNatureIds(newNatures.toArray(new String[newNatures.size()]));
		project.setDescription(description, monitor);
		
//		String resourcesLocation = ResourcesPlugin.getPlugin().getStateLocation().toString();
//		resourcesLocation += File.separatorChar + project.getName();
//		File f = new File(resourcesLocation,"sources.dat");
//		if (f.exists()) { f.delete(); }
	}

}
