package pboweaverplugin.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;

public class WeaverPropertyPage extends PropertyPage {

	//private static final String PATH_TITLE = "&Path:";
	public static final String PATH_PROPERTY = "PATH";
	
	//private static final String DEFAULT_PATH = "/path/to/weaver/executable";
	
	private static final String INST_TITLE = "&Instantiations:";
	public static final String INST_PROPERTY = "INSTANTIATIONS";
	
	private static final String DEFAULT_INSTANTIATION = "";

	private static final int TEXT_FIELD_WIDTH = 50;

	private Text instText;
	//private Text pathText;

	/**
	 * Constructor for SamplePropertyPage.
	 */
	public WeaverPropertyPage() {
		super();
	}

	// With the weaver now being packaged with the plugin, it is no longer necessary
	// to have the user specify the path to the weaver files.
	
	/*
	@Deprecated
	private void addFirstSection(Composite parent) {
		
		Group group = new Group(parent, SWT.NONE);
		group.setText("PbO Weaver Path");
		
		GridData gd;
		
		gd = new GridData(SWT.FILL, SWT.FILL,true, false);
		group.setLayoutData(gd);
		GridLayout gl = new GridLayout();
		group.setLayout(gl);
		
		Label pathLabel = new Label(group, SWT.NONE);
		pathLabel.setText(PATH_TITLE);
		
		pathText = new Text(group, SWT.WRAP |SWT.BORDER | SWT.MULTI);
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.verticalSpan = 2;
		pathText.setLayoutData(gd);
		
		// file browse button
		
		Button button = new Button(group, SWT.PUSH);
		button.setText("Browse");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String file = null;
				
				FileDialog dialog = new FileDialog(getShell(),SWT.OPEN);
				dialog.setFilterPath("~/");
				
				file = dialog.open();
				
				System.out.println("File selected: "+file);
				
				//text = file;
				pathText.setText(file);
			}
		});
		
		gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
		button.setLayoutData(gd);
		
		String text = null;
		try {
			text = ((IResource)getElement()).getPersistentProperty(new QualifiedName("",PATH_PROPERTY));
		} catch (CoreException ex) {
			
		}
		
		pathText.setText((text == null)? "<path to weaver executable>" : text);
		
	}
	*/

	private void addSeparator(Composite parent) {
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		separator.setLayoutData(gridData);
	}

	private void addSecondSection(Composite parent) {
		
		Group group = new Group(parent, SWT.NONE);
		group.setText("PbO Weaver Instantiations");
		
		GridData gd;
		
		gd = new GridData(SWT.FILL, SWT.FILL,true, true);
		group.setLayoutData(gd);
		GridLayout gl = new GridLayout();
		group.setLayout(gl);
		
		Label instLabel = new Label(group, SWT.NONE);
		instLabel.setText(INST_TITLE);
		
		instText = new Text(group, SWT.WRAP |SWT.BORDER | SWT.MULTI);
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		//gd.verticalSpan = 2;
		instText.setLayoutData(gd);
		
		try {
			String text = ((IResource)getElement()).getPersistentProperty(new QualifiedName("",INST_PROPERTY));
			if (text != null) {
				instText.setText(text);
			}
		} catch (CoreException ex) {
			ex.printStackTrace();
		}
		
	}

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		GridData data = new GridData(SWT.FILL);
		data.grabExcessHorizontalSpace = true;
		composite.setLayoutData(data);

		//addFirstSection(composite);
		//addSeparator(composite);
		addSecondSection(composite);
		return composite;
	}

	protected void performDefaults() {
		super.performDefaults();
		// Populate the owner text field with the default value
		instText.setText(DEFAULT_INSTANTIATION);
		//pathText.setText(DEFAULT_PATH);
	}
	
	public boolean performOk() {
		try {
			//String weaverPath = pathText.getText();
			String instantiations = instText.getText();
			
			//File f = new File(weaverPath);
			//if (!f.exists()) { 
			//	MessageDialog.openError(getShell(), "File Not Found", weaverPath+" is not a valid file.  Please check the file path.");
			//	return false;
			//}
			
			final IProject proj = (IProject)getElement();
			
			//proj.setPersistentProperty(new QualifiedName("",PATH_PROPERTY), weaverPath);
			proj.setPersistentProperty(new QualifiedName("",INST_PROPERTY), instantiations);
			
			// run this in the UI thread so that when we click OK, there isn't a stutter as the project 
			// gets rebuilt
			Display.getDefault().asyncExec(new Runnable() {
				@Override
				public void run() {
					try {
						proj.build(IncrementalProjectBuilder.FULL_BUILD, new NullProgressMonitor());
					} catch (CoreException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			});
			
		} catch (CoreException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			
			return false;
		}
		return true;
	}

}