package pboweaverplugin.view;


import java.util.HashMap;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.ViewPart;

import pboweaverplugin.view.constructs.AbstractPbOObject;
import pboweaverplugin.view.model.Location;
import pboweaverplugin.view.model.PbOCategory;
import pboweaverplugin.view.model.PbOModelFactory;


/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */

public class PbOView extends ViewPart {

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "pboweaverplugin.view.PbOView";

	private TreeViewer viewer;
	private Action action1;
	private Action action2;
	private Action doubleClickAction;

	
	
	class NameSorter extends ViewerSorter {
		public int category(Object obj) {
			if (obj instanceof PbOCategory) { return 1; }
			if (obj instanceof AbstractPbOObject) { return 2; }
			if (obj instanceof Location) { return 3; }
			
			return 0;
		}
		
		public int compare(Viewer viewer, Object o1, Object o2) {
			if (o1 instanceof PbOCategory && o2 instanceof PbOCategory) {
				PbOCategory c1 = (PbOCategory)o1;
				PbOCategory c2 = (PbOCategory)o2;
				
				String name1 = c1.getText();
				String name2 = c2.getText();
				
				if (name1.equals("Parameters")) { return -1; }
				if (name2.equals("Parameters")) { return 1; }
				if (name1.equals("Choices")) { return -1; }
				return 1;
				
			}
			
			return super.compare(viewer, o1, o2);
		}
	}

	/**
	 * The constructor.
	 */
	public PbOView() {
		
	}
	
	private void registerListeners() {
		registerActivePageListener();
		registerInfoFileListener();
	}
	
	/*
	 * Handle changing active editor tab
	 * This handles the case where the user switches editors
	 * to a non-pbo project that does not contain the 
	 * pbo.info file.  In that case, we should not display anything
	 * in the view and the ModelFactory will set as input to the view
	 * the EmptyPbOModel
	 */
	// the current project will be dictated by the file opened by the current
	// active editor window
	IProject currentProject = null;
	
	private class ActivePageListener implements IPartListener2 {
		@Override
		public void partActivated(IWorkbenchPartReference partRef) {
			if (viewer != null && !viewer.getControl().isDisposed()) {
				IWorkbenchPage page = partRef.getPage();
				try {
					IFile file = (IFile)page.getActiveEditor().getEditorInput().getAdapter(IFile.class);
				
					if (currentProject == null || file.getProject() != currentProject) {
						currentProject = file.getProject();
						viewer.setInput(PbOModelFactory.createModel(file));
					}
				} catch (NullPointerException ex) {
					currentProject = null;
					viewer.setInput(PbOModelFactory.createModel());
				}

			}
		}

		@Override
		public void partBroughtToTop(IWorkbenchPartReference partRef) { }

		@Override
		public void partClosed(IWorkbenchPartReference partRef) {
			
		}

		@Override
		public void partDeactivated(IWorkbenchPartReference partRef) { }

		@Override
		public void partOpened(IWorkbenchPartReference partRef) { }

		@Override
		public void partHidden(IWorkbenchPartReference partRef) { }

		@Override
		public void partVisible(IWorkbenchPartReference partRef) { }

		@Override
		public void partInputChanged(IWorkbenchPartReference partRef) { }	
	}
	
	private ActivePageListener currentPageListener = new ActivePageListener();
	
	private void registerActivePageListener() {
		try {
			IWorkbench workbench = PlatformUI.getWorkbench();
			IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
			window.getPartService().addPartListener(currentPageListener);
		} catch (NullPointerException ex) {
			System.out.println("Could not register currentPageListener");
		}
	}
	
	private void deregisterActivePageListener() {
		try {
			IWorkbench workbench = PlatformUI.getWorkbench();
			IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
			window.getPartService().removePartListener(currentPageListener);
		} catch (NullPointerException ex) {
			System.out.println("Could not deregister currentPageListener");
		}
	}
	
	private class InfoFileResourceChangeListener implements IResourceChangeListener {

		@Override
		public void resourceChanged(IResourceChangeEvent event) {
			IResourceDelta delta = event.getDelta();
			
			if (delta != null) {
				try {
					delta.accept(new IResourceDeltaVisitor() {

						@Override
						public boolean visit(IResourceDelta delta) throws CoreException {
							IResource resource = delta.getResource();
							final IFile changedFile = (IFile)resource.getAdapter(IFile.class);
							
							if (changedFile != null && (IResourceDelta.CHANGED == delta.getKind()|| IResourceDelta.ADDED == delta.getKind())) {
								if (changedFile.getName().equals("pbo.info") && changedFile.getProject() == currentProject) {
									Display.getDefault().asyncExec(new Runnable() {
										public void run() {
											viewer.setInput(PbOModelFactory.createModel(changedFile));
										}
									});
								}
								
							}
							return true;
						}
						
					});
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}
		
	}
	
	private InfoFileResourceChangeListener infoFileListener = new InfoFileResourceChangeListener();
	
	private void registerInfoFileListener() {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		workspace.addResourceChangeListener(infoFileListener);
	}
	
	private void deregisterInfoFileListener() {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		workspace.removeResourceChangeListener(infoFileListener);
	}
	
	private IFile getActiveFile() {
		try {
			IWorkbench workbench = PlatformUI.getWorkbench();
			IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
			IWorkbenchPage page = window.getActivePage();
			IEditorPart editor = page.getActiveEditor();
			return (IFile)editor.getEditorInput().getAdapter(IFile.class);			
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(PbOModelFactory.createModel(getActiveFile()));
		viewer.setAutoExpandLevel(2);

		// Create the help context id for the viewer's control
		PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(), "PbOView.viewer");
		
		registerListeners();
		
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
	}
	
	public void dispoise() {
		viewer = null;
		deregisterInfoFileListener();
		deregisterActivePageListener();
		
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				PbOView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		// At the moment we do not need actions
		
		//manager.add(action1);
		//manager.add(new Separator());
		//manager.add(action2);
	}

	private void fillContextMenu(IMenuManager manager) {
		// At the moment we do not need actions
		
		//manager.add(action1);
		//manager.add(action2);
		// Other plug-ins can contribute there actions here
		//manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		// At the moment we do not need these actions
		//manager.add(action1);
		//manager.add(action2);
	}

	private void makeActions() {
		action1 = new Action() {
			public void run() {
				showMessage("Action 1 executed");
			}
		};
		action1.setText("Action 1");
		action1.setToolTipText("Action 1 tooltip");
		action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
		
		action2 = new Action() {
			public void run() {
				showMessage("Action 2 executed");
			}
		};
		action2.setText("Action 2");
		action2.setToolTipText("Action 2 tooltip");
		action2.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
				getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
		doubleClickAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				final Object obj = ((IStructuredSelection)selection).getFirstElement();
				//showMessage("Double-click detected on "+obj.toString());
				
				Display.getDefault().asyncExec(new Runnable() {

					@Override
					public void run() {
						Location location = null;;
						
						if (obj instanceof AbstractPbOObject) {
							AbstractPbOObject pboObj = (AbstractPbOObject)obj;
							location = pboObj.getDeclared();	
						} else if (obj instanceof Location) {
							location = (Location)obj;
						}
						
						if (location == null) return;
						
						int lineNumber = location.getLineNumber();
						IFile filename = location.getFile();
						
						IWorkbench workbench = PlatformUI.getWorkbench();
						IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
						IWorkbenchPage page = window.getActivePage();
						
						HashMap<String,Object> map = new HashMap<String, Object>();
						map.put(IMarker.LINE_NUMBER, lineNumber);
						
						try {
							IMarker marker = filename.createMarker(IMarker.TEXT);
							marker.setAttributes(map);
							IDE.openEditor(page, marker);
							marker.delete();
						} catch (PartInitException e) {
							e.printStackTrace();
						} catch (CoreException e) {
							e.printStackTrace();
						}
						
					}
					
				});
			}
		};
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
	private void showMessage(String message) {
		MessageDialog.openInformation(
			viewer.getControl().getShell(),
			"PbO View",
			message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
}
