package pboweaverplugin.view;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;

import pboweaverplugin.view.constructs.AbstractPbOObject;
import pboweaverplugin.view.model.IPbOModel;
import pboweaverplugin.view.model.PbOCategory;

/*
 * The content provider class is responsible for
 * providing objects to the view. It can wrap
 * existing objects in adapters or simply return
 * objects as-is. These objects may be sensitive
 * to the current input of the view, or ignore
 * it and always show the same content 
 * (like Task List, for example).
 */
 
public class ViewContentProvider implements ITreeContentProvider {
	private IPbOModel model;
	
	@Override
	public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		this.model = (IPbOModel)newInput;
	}
	
	@Override
	public Object[] getElements(Object parent) {
		return model.getPbOConstructs().toArray();
	}

	@Override
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof PbOCategory) {
			PbOCategory category = (PbOCategory)parentElement;
			return category.getItems().toArray();
		}
		
		if (parentElement instanceof AbstractPbOObject) {
			AbstractPbOObject object = (AbstractPbOObject)parentElement;
			return object.getAccessed().toArray();
		}
		return null;
	}

	@Override
	public Object getParent(Object element) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {
		if (element instanceof PbOCategory) {
			PbOCategory category = (PbOCategory)element;
			return category.getItems().size() > 0;
		}
		
		if (element instanceof AbstractPbOObject) {
			AbstractPbOObject obj = (AbstractPbOObject)element;
			return obj.getAccessed().size() > 0;
		}
		
		return false;
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub
		
	}
}