package pboweaverplugin.view.parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;

import pboweaverplugin.view.constructs.AbstractPbOObject;
import pboweaverplugin.view.constructs.PbOChoiceBuilder;
import pboweaverplugin.view.constructs.PbOLogBuilder;
import pboweaverplugin.view.constructs.PbOParamBuilder;
import pboweaverplugin.view.parser.fsm.choice.PbOChoiceFiniteStateMachine;
import pboweaverplugin.view.parser.fsm.log.PbOLogFiniteStateMachine;
import pboweaverplugin.view.parser.fsm.parameter.PbOParameterFiniteStateMachine;

public class PbOInfoParser {
	IFile infoFile;
	
	ParameterParserState paramState;
	ChoiceParserState choiceState;
	LogParserState logState;
	EmptyParserState emptyState;
	
	InfoParserState currentState;
	
	public PbOInfoParser(IFile infoFile, PbOParamBuilder paramBuilder, PbOChoiceBuilder choiceBuilder, PbOLogBuilder logBuilder) {
		this.infoFile = infoFile;
		
		PbOParameterFiniteStateMachine paramFSM = new PbOParameterFiniteStateMachine(paramBuilder);
		PbOChoiceFiniteStateMachine choiceFSM = new PbOChoiceFiniteStateMachine(choiceBuilder);
		PbOLogFiniteStateMachine logFSM = new PbOLogFiniteStateMachine(logBuilder);
		
		paramState = new ParameterParserState(paramFSM);
		choiceState = new ChoiceParserState(choiceFSM);
		logState = new LogParserState(logFSM);
		emptyState = new EmptyParserState(null);
		
		currentState = paramState;
	}
	
	public void parse() throws CoreException, IOException {
		InputStreamReader isr = new InputStreamReader(infoFile.getContents());
		BufferedReader br = new BufferedReader(isr);
		
		String line = null;
		
		while((line = br.readLine()) != null) {
			
			if (line.matches("//\\s*parameters\\s*")) {
				currentState = paramState;
				continue;
			}
			
			if (line.matches("//\\s*choices\\s*")) {
				currentState = choiceState;
				continue;
			}
			
			if (line.matches("//\\s*logs\\s*")) {
				currentState = logState;
				continue;
			}
			
			currentState.parse(line);
		}
	}
	
	public List<AbstractPbOObject> getParameters() {
		return paramState.getParameters();
	}
	
	public List<AbstractPbOObject> getChoices() {
		return choiceState.getChoices();
	}
	
	public List<AbstractPbOObject> getLogs() {
		return logState.getLogs();
	}
	
}
