package pboweaverplugin.view.parser.fsm;

import java.util.ArrayList;
import java.util.List;

import pboweaverplugin.view.constructs.AbstractPbOBuilder;
import pboweaverplugin.view.constructs.AbstractPbOObject;

public abstract class PbOParserFiniteStateMachine {
	AbstractPbOBuilder builder;
	List<AbstractPbOObject> list;
	
	/**
	 * Constructor that takes in an AbstractPbOBuilder which represents
	 * the kind of PbO Object to construct
	 * 
	 * @param builder The kind of PbO Object this Finite State Machine should construct
	 */
	public PbOParserFiniteStateMachine(AbstractPbOBuilder builder) {
		this.builder = builder;
		list = new ArrayList<AbstractPbOObject>();
	}
	
	public AbstractPbOBuilder getBuilder() {
		return builder;
	}
	
	/**
	 * Add to this Finite State Machine an already built PbO Object built by the builder
	 * @param obj
	 */
	protected void addObject(AbstractPbOObject obj) {
		list.add(obj);
	}
	
	/**
	 * Returns a list of the constructed PbO Objects this Finite State Machine is responsible
	 * for building
	 */
	protected List<AbstractPbOObject> getObjects() {
		return list;
	}
	
	/**
	 * Parse a string for information to feed to the builder or to transition to another state
	 * @param line
	 */
	public abstract void parse(String line);
}
