package ca.ubc.cs.beta.pbo.ast;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.cdt.core.dom.ast.IASTDeclSpecifier;
import org.eclipse.cdt.core.dom.ast.IASTDeclarator;
import org.eclipse.cdt.core.dom.ast.IASTExpression;
import org.eclipse.cdt.core.dom.ast.IASTName;
import org.eclipse.cdt.core.dom.ast.IASTSimpleDeclSpecifier;
import org.eclipse.cdt.core.dom.ast.IASTSimpleDeclaration;
import org.eclipse.cdt.core.dom.ast.IASTStatement;
import org.eclipse.cdt.core.dom.ast.IBasicType;
import org.eclipse.cdt.core.dom.ast.INodeFactory;
import org.eclipse.cdt.core.dom.ast.c.ICNodeFactory;
import org.eclipse.cdt.core.parser.EndOfFileException;
import org.eclipse.cdt.core.parser.IProblem;
import org.eclipse.cdt.core.parser.IToken;
import org.eclipse.cdt.internal.core.dom.parser.ASTNode;
import org.eclipse.cdt.internal.core.dom.parser.BacktrackException;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTName;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTPbOParamDeclarator;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTPbOParamExpression;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTPbOStatement;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTProblem;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTSimpleDeclSpecifier;
import org.eclipse.cdt.internal.core.dom.parser.c.CASTSimpleDeclaration;

public class PBOParserUtil {

	
	public static IASTSimpleDeclaration parsePBODeclaration(PbOExporterInterface pboExporter) throws BacktrackException, EndOfFileException
	{
		
		
		int initialOffset = 0;
		//NOT SURE WHY I SEE TWO PBOTokens
		IToken t = pboExporter.getLookAheadNext();
		initialOffset = t.getOffset();
		t = pboExporter.getLookAheadNext();
		/*
		for(int i=0; i < 10; i++)
		{
			IToken ta = pboExporter.LookAhead(i);
			System.out.println("Token Scan:" + ta.getImage() + " : " + ta.getType() );
		}*/
		
		IASTSimpleDeclaration result;
		if(t.getImage().equals("PARAM"))
		{
			result = parsePARAMDeclaration(pboExporter, initialOffset);
			
		} else if(t.getImage().equals("BEGIN"))
		{
			result = parseCHOICEDeclaration(pboExporter, initialOffset,true, false);
		} else if(t.getImage().equals("END"))
		{
			result = parseCHOICEDeclaration(pboExporter, initialOffset,false, false);
		} else if(t.getImage().equals("CHOICE"))
		{
			pboExporter.backup();
			result = parseCHOICEDeclaration(pboExporter, initialOffset,true, true);
		
		} else	
		{
			throw new BacktrackException();
		}
		
		
		for(int i=1; i < pboExporter.getLookAheadCount(); i++)
		{
			pboExporter.consume();
		}
		
		return result;
		
	}
	
	private static IASTSimpleDeclaration parseCHOICEDeclaration(PbOExporterInterface pboExporter, int initialOffset, boolean allowLiteral, boolean requireLiteral) throws EndOfFileException, BacktrackException {
		IToken t = pboExporter.getLookAheadNext();
		
		//Stub code to test error reporting, not sure why the error doesn't appear
		CASTProblem prob = new CASTProblem(IProblem.PREPROCESSOR_INVALID_DIRECTIVE, "Error of some kind".toCharArray(), true);
		prob.setOffsetAndLength(t.getOffset(), t.getLength());
		BacktrackException exp = new BacktrackException();
		exp.initialize(prob);
	
		if(!t.getImage().equals("CHOICE")) throw exp;

		t = pboExporter.getLookAheadNext(); 
		IToken name = t;
		if(t.getType() != IToken.tIDENTIFIER) throw new BacktrackException();
		if(pboKeyword(name)) throw new BacktrackException();
		
		
		IToken lastToken = t;
		String literalName = "";
		if(allowLiteral)
		{
			t = pboExporter.getLookAheadNext();
			switch(t.getType())
			{	
				case IToken.tASSIGN:
					
					
					t = pboExporter.getLookAheadNext();
					//validateLiteral(pboExporter, t.getType());
					validateNamedChoiceInstance(pboExporter, t.getType());
					literalName = "=" + t.getImage();
					lastToken = t;
					
					
					break;
				default:
					pboExporter.backup();
					
			} 
		} else 
		{
			if(requireLiteral)
			{
				throw new BacktrackException(); 
			}
			//We won't need to check if a literal was given, and shouldn't have been as the AST will later parse.
		}
		INodeFactory nodeFactory = pboExporter.getNodeFactory();
		IASTDeclSpecifier pbodeclspec= nodeFactory.newSimpleDeclSpecifier();
    	IASTSimpleDeclaration pbodecl= nodeFactory.newSimpleDeclaration(pbodeclspec);
    	
    	
    	//& 0 in the next line is because for some reason Code Analysis Detector 
    	//complains about extern variables being unused. I'm not sure why
    	//But I think this might have to be sc_extern so just remove the & 0 if this is the case
 		pboExporter.configureDeclSpec(pbodeclspec,IASTDeclSpecifier.sc_extern, pboExporter.getConst() );
 		
 		((IASTSimpleDeclSpecifier) pbodeclspec).setType(IBasicType.Kind.eUnspecified);
 		
 		IASTName name2 = pboExporter.getName(name, literalName);
 		final IASTDeclarator pboDeclaration = new CASTPbOParamDeclarator(name2);
 		
 		((IASTSimpleDeclaration) pbodecl).addDeclarator(pboDeclaration);
 		
 		
 		
 		((ASTNode)pbodecl).setLength(lastToken.getEndOffset()-initialOffset);
		
		
		
	
		return pbodecl;
		
		
		
	}
	
	private static IASTSimpleDeclaration parsePARAMDeclaration(PbOExporterInterface pboExporter, int initialOffset) throws EndOfFileException, BacktrackException {
		
		if(pboExporter.getLookAheadNext().getType() != IToken.tLPAREN) throw new BacktrackException();
		IToken t = pboExporter.getLookAheadNext();
		
		IToken type = t;
		if(!validPBOType(t)) throw new BacktrackException();
		
		t = pboExporter.getLookAheadNext();
		
		switch(t.getType())
		{
			case IToken.tLBRACKET:
				//Array Notation
				int next = pboExporter.getLookAheadNext().getType();
				validateNumericLiteral(next);
				
				if(pboExporter.getLookAheadNext().getType() != IToken.tCOMMA) throw new BacktrackException();
				next = pboExporter.getLookAheadNext().getType();
				validateNumericLiteral(next);
				
				if(pboExporter.getLookAheadNext().getType() != IToken.tRBRACKET) throw new BacktrackException();
				t = pboExporter.getLookAheadNext();
				
				
			break;
			case IToken.tLBRACE:
				
				do { 
					next = pboExporter.getLookAheadNext().getType();
					
					validateLiteral(pboExporter, next);
					next = pboExporter.getLookAheadNext().getType();
					if(next == IToken.tCOMMA)
					{
						continue;
					}
					if(next == IToken.tRBRACE)
					{
						break;
					}
										
				} while(true);
			
				t = pboExporter.getLookAheadNext();
				
			default:
					break;
		}
		
		
		IToken name = t;
		if(t.getType() != IToken.tIDENTIFIER) throw new BacktrackException();
		if(pboKeyword(name)) throw new BacktrackException();
		
		t = pboExporter.getLookAheadNext();
		
		IToken lastToken;
		switch(t.getType())
		{
			case IToken.tRPAREN:
				lastToken = t;
				break;
			case IToken.tEQUAL:
			case IToken.tASSIGN:
				
				validateLiteral(pboExporter, pboExporter.getLookAheadNext().getType());
				
				t = pboExporter.getLookAheadNext();
				lastToken = t;
				if(t.getType() != IToken.tRPAREN) throw new BacktrackException();
				
				break;
			default:
				throw new BacktrackException();
		}
		
		INodeFactory nodeFactory = pboExporter.getNodeFactory();
		IASTDeclSpecifier pbodeclspec= nodeFactory.newSimpleDeclSpecifier();
    	IASTSimpleDeclaration pbodecl= nodeFactory.newSimpleDeclaration(pbodeclspec);
    	
    	//& 0 in the next line is because for some reason Code Analysis Detector 
    	//complains about extern variables being unused. I'm not sure why
    	//But I think this might have to be sc_extern so just remove the & 0 if this is the case
 		pboExporter.configureDeclSpec(pbodeclspec,IASTDeclSpecifier.sc_extern, pboExporter.getConst() );
 		
 		((IASTSimpleDeclSpecifier) pbodeclspec).setType(getKind(type.getImage()));
 	
 		
 		IASTName name2 = pboExporter.getName(name,"");

 		final IASTDeclarator pboDeclaration = new CASTPbOParamDeclarator(name2);
 		
 		((IASTSimpleDeclaration) pbodecl).addDeclarator(pboDeclaration);
 		
 		
 		
 		((ASTNode)pbodecl).setLength(lastToken.getEndOffset()-initialOffset);
		
		
		
	
		return pbodecl;
		
	}
	
	private static final Set<String> types;
	private static final Set<String> pboKeywords;
	static 
	{
		String[] validTypes = {"bool","boolean","int","integer","float","real","char","character","str","string"};
		String[] pboKeywordArray = {"LOG","BEGIN", "END","CHOICE","PARAM"};
		types = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(validTypes)));
		pboKeywords = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(pboKeywordArray)));
	}
	
	private static boolean validPBOType(IToken t)
	{		
		return types.contains(t.getImage());
	}
	
	private static boolean pboKeyword(IToken t)
	{
		return pboKeywords.contains(t.getImage());
	}
	
	private static IBasicType.Kind getKind(String s) throws BacktrackException
	{
		if(s.equals("bool")) return IBasicType.Kind.eBoolean;
		if(s.equals("boolean")) return IBasicType.Kind.eBoolean;
		if(s.equals("int")) return IBasicType.Kind.eInt;
		if(s.equals("integer")) return IBasicType.Kind.eInt;
		if(s.equals("float")) return IBasicType.Kind.eFloat;
		if(s.equals("real")) return IBasicType.Kind.eFloat;
		if(s.equals("char")) return IBasicType.Kind.eChar;
		if(s.equals("character")) return IBasicType.Kind.eChar;
		if(s.equals("str")) return IBasicType.Kind.eChar;
		if(s.equals("string")) return IBasicType.Kind.eChar;
		
		throw new BacktrackException();
	}
	
	private static void validateNumericLiteral(int type) throws BacktrackException
	{
		switch(type)
		{
			case IToken.tINTEGER:
			case IToken.tFLOATINGPT:
				break;
			default:
				throw new BacktrackException();
		}
	}
	
	private static void validateLiteral(PbOExporterInterface pboExporter, int next) throws BacktrackException, EndOfFileException {
		switch(next)
		{	
			
			case IToken.tMINUS:
			case IToken.tPLUS:
				validateLiteral(pboExporter, pboExporter.getLookAheadNext().getType());
				break;
			case IToken.tINTEGER:
			case IToken.tFLOATINGPT:
			case IToken.tCHAR:
			case IToken.tSTRING:
				break;
			default:
				throw new BacktrackException();
		}
		
		
	}
	
	private static void validateNamedChoiceInstance(PbOExporterInterface pboExporter, int next) throws BacktrackException, EndOfFileException {
		switch(next) {
			case IToken.tMINUS:
			case IToken.tPLUS:
				validateNamedChoiceInstance(pboExporter, pboExporter.getLookAheadNext().getType());
				break;
			case IToken.tINTEGER:
			case IToken.tIDENTIFIER:
				break;
			default:
				throw new BacktrackException();
		}
	}

	public static IASTStatement parsePBOLogStatement(PbOExporterInterface pboExporter) throws EndOfFileException, BacktrackException
	{
		
		IToken markEnd= pboExporter.mark();
		
		/*
		try {
			IASTExpression expr= primaryExpression(ctx, strat);
			IASTFunctionCallExpression fcall = nodeFactory.newFunctionCallExpression(expr, (IASTExpression[]) null);
			IASTAmbiguousExpression ambiguity = createAmbiguousCastVsFunctionCallExpression(result, fcall);
			((ASTNode) ambiguity).setOffsetAndLength((ASTNode) result);
			return ca == null ? ambiguity : ca.updateExpression(ambiguity);
		} catch (BacktrackException bt) {
		} finally {
			backup(markEnd);
		}
		*/
		 
			
		
		int initialOffset = 0;
		//NOT SURE WHY I SEE TWO PBOTokens
		IToken t = pboExporter.getLookAheadNext();
		initialOffset = t.getOffset();
		t = pboExporter.getLookAheadNext();
	
		if(!t.getImage().equals("LOG")) throw new BacktrackException();
		t = pboExporter.getLookAheadNext();
		
		if(t.getType() != IToken.tLPAREN) throw new BacktrackException();
		
		t = pboExporter.getLookAheadNext();
		if(!validPBOType(t)) throw new BacktrackException();
		
		t = pboExporter.getLookAheadNext();
		if((t.getType() != IToken.tIDENTIFIER) || pboKeyword(t)) throw new BacktrackException();
		
		
		t = pboExporter.getLookAheadNext();
		if(t.getType() != IToken.tASSIGN) throw new BacktrackException();
		
		
		t = pboExporter.getLookAheadNext();
		if(t.getType() == IToken.tRPAREN) throw new BacktrackException();
		pboExporter.backup();
		
		int rParensToFind = 1;
		do
		{
			t = pboExporter.getLookAheadNext();
			if(t.getType() == IToken.tLPAREN)
				rParensToFind++;
			if(t.getType() == IToken.tRPAREN)
				rParensToFind--;
		} while(rParensToFind > 0);
			
		
		
		
		
		
		CASTPbOStatement pbo = new CASTPbOStatement();
		pboExporter.consume();
		pbo.setOffsetAndLength(initialOffset, t.getEndOffset() - initialOffset);
		
			
		for(int i=2; i < pboExporter.getLookAheadCount(); i++)
		{
			pboExporter.consume();
		}
		return pbo;
		
		
		
		
		
	}
	public static IASTExpression parsePBOPrimaryExpression(PbOExporterInterface pboExporter) throws EndOfFileException, BacktrackException {
		// 
		int initialOffset = 0;
		//NOT SURE WHY I SEE TWO PBOTokens
		IToken t = pboExporter.getLookAheadNext();
		initialOffset = t.getOffset();
		t = pboExporter.getLookAheadNext();

		CASTPbOParamExpression pbo = new  CASTPbOParamExpression();
		
		pbo.setOffsetAndLength(initialOffset, t.getEndOffset() - initialOffset);
		
		for(int i=1; i < pboExporter.getLookAheadCount(); i++)
		{
			pboExporter.consume();
		}
		
		
		if((t.getType() != IToken.tIDENTIFIER) || pboKeyword(t))
		{
			//Stub code to test error reporting, not sure why the error doesn't appear
			CASTProblem prob = new CASTProblem(IProblem.PREPROCESSOR_INVALID_DIRECTIVE, "Error of some kind".toCharArray(), true);
			prob.setOffsetAndLength(t.getOffset(), t.getLength());
			BacktrackException exp = new BacktrackException();
			exp.initialize(prob);
			prob.setParent(pbo);
			throw new BacktrackException();
			
		}
		
		 return pbo;

	}
}
