package ca.ubc.cs.beta.pbo.eclipse.editor;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWhitespaceDetector;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.PatternRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;



/**
 * Used to tokenize & color PBO Partitions 
 * @author seramage
 *
 */
public abstract class PBOScanner extends RuleBasedScanner {

			private RGB javaKeyword = new RGB(127,0,85);
			private List<IToken> prevTokens = new LinkedList<IToken>();
			
			
			protected abstract Color getColor(RGB g);
			
			public PBOScanner() {
				
				
			
			}
			
			/**
			 * Children should call this method after getColor() is callable 
			 */
			protected void init()
			{
				System.out.println("Scanner Initialized");
				Font font = new Font(Display.getCurrent(),"Arial",10,SWT.BOLD); 
				
				IToken paramDeclaration = new Token(new TextAttribute(getColor(new RGB(255, 0, 0))));
				IToken logDeclaration = new Token(new TextAttribute(getColor(new RGB(34,139,34))));
				IToken choiceDeclaration = new Token(new TextAttribute(getColor(new RGB(0, 0, 255))));
				IToken paramReadDeclaration = new Token(new TextAttribute(getColor(new RGB(255, 0, 255))));
				IToken defaultBlack = new Token(new TextAttribute(getColor(new RGB(0, 0, 0))));
				
				
				IToken typeDeclaration = new Token(new TextAttribute(getColor(javaKeyword),null,0, font));
				this.setDefaultReturnToken(paramReadDeclaration);
				
				List<IRule> rules = new LinkedList<IRule>();
				
				
				//Adds rules for types
				
				rules.add(new ExactMatchRule("##PARAM",paramDeclaration));
				rules.add(new ExactMatchRule("##LOG",logDeclaration));
				rules.add(new RegExMatcher("\\A##(BEGIN\\s|END\\s|)CHOICE\\s+\\w+\\s*(=\\s*\\w+)?",choiceDeclaration,70));
				rules.add(new RegExMatcher("\\A=",defaultBlack,2));
				
				rules.add(new RegExMatcher("\\A##\\S+",paramReadDeclaration,50)); //Read 50 Characters
				
				
				
				for(String type : TypeRule.validTypes)
				{
					rules.add(new RegExMatcher("\\A" + type +"\\s",typeDeclaration,10));
				}
				
				
				
				//rules.add(new LineContainsRule("CHOICE",choiceDeclaration));
				

				
				setRules(rules.toArray(new IRule[0]));
				
			}

		public class JavaIdentifierWordDetector implements IWordDetector {
			public boolean isWordPart(char character) {
				return Character.isJavaIdentifierPart(character);
			}
			public boolean isWordStart(char character) {
				return Character.isJavaIdentifierStart(character);
			}
		}
		
		@Override
		public IToken nextToken()
		{
			return super.nextToken();
		}


}